

#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>

#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "app_button.h"
#include "app_gpiote.h"
#include "app_timer.h"
#include "app_util_platform.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "boards.h"
#include "softdevice_handler.h"
#include "app_timer.h"
#include "fstorage.h"
#include "fds.h"
#include "peer_manager.h"
#include "nrf_delay.h" 
#include "nrf_drv_power.h"
#include "nrf_drv_twi.h"

#include "bsp.h"
#include "bsp_btn_ble.h"
#include "sensorsim.h"
#include "nrf_gpio.h"
#include "nrf_drv_adc.h"
#include "app_timer.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_state.h"
#include "system.h" 


#define NRF_LOG_MODULE_NAME "T_APP"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"

#define IS_SRVC_CHANGED_CHARACT_PRESENT 1                                           /**< Include or not the service_changed characteristic. if not enabled, the server's database cannot be changed for the lifetime of the device*/

#if (NRF_SD_BLE_API_VERSION == 3)
#define NRF_BLE_MAX_MTU_SIZE            GATT_MTU_SIZE_DEFAULT                       /**< MTU size used in the softdevice enabling and to reply to a BLE_GATTS_EVT_EXCHANGE_MTU_REQUEST event. */
#endif

#define APP_FEATURE_NOT_SUPPORTED       BLE_GATT_STATUS_ATTERR_APP_BEGIN + 2        /**< Reply when unsupported features are requested. */

#define CENTRAL_LINK_COUNT              0                                           /**< Number of central links used by the application. When changing this number remember to adjust the RAM settings*/
#define PERIPHERAL_LINK_COUNT           1                                           /**< Number of peripheral links used by the application. When changing this number remember to adjust the RAM settings*/

#define DEVICE_NAME                     "Nordic_Template"                           /**< Name of device. Will be included in the advertising data. */
#define MANUFACTURER_NAME               "NordicSemiconductor"                       /**< Manufacturer. Will be passed to Device Information Service. */
#define APP_ADV_INTERVAL                300                                         /**< The advertising interval (in units of 0.625 ms. This value corresponds to 187.5 ms). */
#define APP_ADV_TIMEOUT_IN_SECONDS      10                                         /**< The advertising timeout in units of seconds. */

#define APP_TIMER_PRESCALER             0                                           /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_OP_QUEUE_SIZE         4                                           /**< Size of timer operation queues. */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(100, UNIT_1_25_MS)            /**< Minimum acceptable connection interval (0.1 seconds). */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(200, UNIT_1_25_MS)            /**< Maximum acceptable connection interval (0.2 second). */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)             /**< Connection supervisory timeout (4 seconds). */

#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000, APP_TIMER_PRESCALER)  /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000, APP_TIMER_PRESCALER) /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                           /**< Number of attempts before giving up the connection parameter negotiation. */

#define SEC_PARAM_BOND                  1                                           /**< Perform bonding. */
#define SEC_PARAM_MITM                  0                                           /**< Man In The Middle protection not required. */
#define SEC_PARAM_LESC                  0                                           /**< LE Secure Connections not enabled. */
#define SEC_PARAM_KEYPRESS              0                                           /**< Keypress notifications not enabled. */
#define SEC_PARAM_IO_CAPABILITIES       BLE_GAP_IO_CAPS_NONE                        /**< No I/O capabilities. */
#define SEC_PARAM_OOB                   0                                           /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE          7                                           /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE          16                                          /**< Maximum encryption key size. */

#define BUTTON_DEBOUNCE_DELAY   				50	//Define button debounce delay
#define APP_GPIOTE_MAX_USERS            1

#define MODE_STBY												0
#define MODE_T1													1
#define MODE_T2													2
#define MODE_T3													3

/* TWI instance ID. */
#define TWI_INSTANCE_ID     						0
/* Number of possible TWI addresses. */
#define TWI_ADDRESSES      						127

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

/* TWI instance. */
static const nrf_drv_twi_t m_twi = NRF_DRV_TWI_INSTANCE(TWI_INSTANCE_ID);

static uint16_t m_conn_handle = BLE_CONN_HANDLE_INVALID;                            /**< Handle of the current connection. */
//static app_timer_id_t                   m_bat_timer_id;                     			  /**< Battery timer. */
//static app_timer_id_t                   timer_16hz;																	/**<16 hertz timer. */

static bool button_power_5sec_timer_enabled = false; //flag for monitoring timer
static bool button_a_5sec_timer_enabled = false; //flag for monitoring timer

APP_TIMER_DEF(button_power_5sec_timer_id); //define 5 second timer id
APP_TIMER_DEF(button_a_5sec_timer_id); //define 5 second timer id


// YOUR_JOB: Use UUIDs for service(s) used in your application.
static ble_uuid_t m_adv_uuids[] = {{BLE_UUID_DEVICE_INFORMATION_SERVICE, BLE_UUID_TYPE_BLE}}; /**< Universally unique service identifiers. */

static void advertising_start(void);
static void button_power_5sec_handler(void *);
static void button_a_5sec_handler(void *);
void change_op_mode(void);



/**@brief Function for the Timer initialization.
 *
 * @details Initializes the timer module. This creates and starts application timers.
 */
static void timers_init(void)
{
    // Initialize timer module.
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);
    // Create timers.
    /* YOUR_JOB: Create any timers to be used by the application.
                 Below is an example of how to create a timer.
                 For every new timer needed, increase the value of the macro APP_TIMER_MAX_TIMERS by
                 one.
     uint32_t err_code;
     err_code = app_timer_create(&m_app_timer_id, APP_TIMER_MODE_REPEATED, timer_timeout_handler);
     APP_ERROR_CHECK(err_code); */
	uint32_t err_code;		
	err_code = app_timer_create(&button_power_5sec_timer_id, APP_TIMER_MODE_SINGLE_SHOT, button_power_5sec_handler);
	APP_ERROR_CHECK(err_code); 
	err_code = app_timer_create(&button_a_5sec_timer_id, APP_TIMER_MODE_SINGLE_SHOT, button_a_5sec_handler);
	APP_ERROR_CHECK(err_code); 	

}



/**@brief Function for handling button.
 *
 * 
 */
static void button_handler(uint8_t pin_no, uint8_t button_action)
{ 
		uint32_t err_code;
    switch(button_action) {
			case APP_BUTTON_PUSH:	
				NRF_LOG_INFO("Button%d Pressed \r\n", pin_no);
				switch(pin_no)
        {
            case BUTTON_POWER:
								app_timer_stop(button_power_5sec_timer_id);                
								err_code = app_timer_start(button_power_5sec_timer_id, APP_TIMER_TICKS(5000,APP_TIMER_PRESCALER), NULL); //start timer, 5 second timeout
								APP_ERROR_CHECK(err_code);
								button_power_5sec_timer_enabled=true;
                break;
            case BUTTON_A:
								app_timer_stop(button_a_5sec_timer_id);
								err_code = app_timer_start(button_a_5sec_timer_id, APP_TIMER_TICKS(5000,APP_TIMER_PRESCALER), NULL); //start timer, 5 second timeout
								APP_ERROR_CHECK(err_code);
								button_a_5sec_timer_enabled=true;						
                break;
            default:
                break;
        }
				break;
			case APP_BUTTON_RELEASE:				
				NRF_LOG_INFO("Button%d Released \r\n", pin_no);
				switch(pin_no)
        {
						case BUTTON_POWER:
								app_timer_stop(button_power_5sec_timer_id);
								button_power_5sec_timer_enabled=false;								
								break;
						case BUTTON_A:
								app_timer_stop(button_a_5sec_timer_id);
								button_a_5sec_timer_enabled=false;
								change_op_mode();
								break;
				}
				break;
			default:
				break;
		}
}



//APP timer timeout handler
static void button_power_5sec_handler(void * p_context)
{
    UNUSED_PARAMETER(p_context);    
    if(nrf_gpio_pin_read(BUTTON_POWER)==1) //if button 1 is still pressed
    {
        NRF_LOG_INFO("Button1 5sec long pressed\r\n");
    }
		app_timer_stop(button_power_5sec_timer_id);
    button_power_5sec_timer_enabled=false;
}

static void button_a_5sec_handler(void * p_context)
{
    UNUSED_PARAMETER(p_context);
    if(nrf_gpio_pin_read(BUTTON_A)==0) //if button 2 is still pressed
    {
        NRF_LOG_INFO("Button2 5sec long pressed\r\n");
				
    }
		app_timer_stop(button_a_5sec_timer_id);
    button_a_5sec_timer_enabled=false;

}



/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init(bool * p_erase_bonds)
{
    bsp_event_t startup_event;
		uint32_t err_code;
		
    /*uint32_t err_code = bsp_init(BSP_INIT_LED | BSP_INIT_BUTTONS,
                                 APP_TIMER_TICKS(100, APP_TIMER_PRESCALER),
                                 bsp_event_handler);*/
		/*
		uint32_t err_code = bsp_init(BSP_INIT_NONE | BSP_INIT_BUTTONS,
                                 APP_TIMER_TICKS(100, APP_TIMER_PRESCALER),
                                 bsp_event_handler);
		
    APP_ERROR_CHECK(err_code);
		*/
		static app_button_cfg_t p_button[] = {{BUTTON_POWER, APP_BUTTON_ACTIVE_HIGH, NRF_GPIO_PIN_NOPULL, button_handler},
                                          {BUTTON_A, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_PULLUP, button_handler}};	
		
		
		// Macro for initializing the GPIOTE module.
    // It will handle dimensioning and allocation of the memory buffer required by the module, making sure that the buffer is correctly aligned.
		APP_GPIOTE_INIT(APP_GPIOTE_MAX_USERS);
																					
		// Initializing the buttons.
    err_code = app_button_init(p_button, sizeof(p_button) / sizeof(p_button[0]), BUTTON_DEBOUNCE_DELAY);
																					
    APP_ERROR_CHECK(err_code);
																					
    err_code = app_button_enable();
    APP_ERROR_CHECK(err_code); 
																				
																					
		err_code = bsp_btn_ble_init(NULL, &startup_event);
    APP_ERROR_CHECK(err_code);	
		
    *p_erase_bonds = (startup_event == BSP_EVENT_CLEAR_BONDING_DATA);
}



/**@brief Function for the Power manager.
 */
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();

    APP_ERROR_CHECK(err_code);
}


static void log_resetreason(void)
{
    /* Reset reason */
    uint32_t rr = nrf_power_resetreas_get();
    NRF_LOG_INFO("Reset reasons:\r\n");
    if(0 == rr)
    {
        NRF_LOG_INFO("- NONE\r\n");
    }
    if(0 != (rr & NRF_POWER_RESETREAS_RESETPIN_MASK))
    {
        NRF_LOG_INFO("- RESETPIN\r\n");
    }
    if(0 != (rr & NRF_POWER_RESETREAS_DOG_MASK     ))
    {
        NRF_LOG_INFO("- DOG\r\n");
    }
    if(0 != (rr & NRF_POWER_RESETREAS_SREQ_MASK    ))
    {
        NRF_LOG_INFO("- SREQ\r\n");
    }
    if(0 != (rr & NRF_POWER_RESETREAS_LOCKUP_MASK  ))
    {
        NRF_LOG_INFO("- LOCKUP\r\n");
    }
    if(0 != (rr & NRF_POWER_RESETREAS_OFF_MASK     ))
    {
        NRF_LOG_INFO("- OFF\r\n");
    }
    if(0 != (rr & NRF_POWER_RESETREAS_LPCOMP_MASK  ))
    {
        NRF_LOG_INFO("- LPCOMP\r\n");
    }
    if(0 != (rr & NRF_POWER_RESETREAS_DIF_MASK     ))
    {
        NRF_LOG_INFO("- DIF\r\n");
    }    
}


/**@brief Function for application main entry.
 */
int main(void)
{
    uint32_t err_code;
    uint8_t i2c_address;
		uint8_t sample_data;
		bool detected_device = false;
		bool     erase_bonds;

    // Initialize.
    err_code = NRF_LOG_INIT(NULL);		
		APP_ERROR_CHECK(err_code);
		
		batt_oldstat=false;
		battlow=false;	
		batt_ledflag = true;	    

	  log_resetreason();
    nrf_power_resetreas_clear(nrf_power_resetreas_get());
	

		tcm_power_init(); //initialize the power related IO pins
		init_leds();	
    timers_init(); //initialize two system timers
    buttons_leds_init(&erase_bonds);

		NRF_LOG_INFO("Template started\r\n");
		adc_sampling_event_init();
    adc_config();
    adc_sampling_event_enable();		
		
		
    ble_stack_init();
    peer_manager_init(erase_bonds);
    if (erase_bonds == true)
    {
		        NRF_LOG_INFO("Bonds erased!\r\n");
    }
    gap_params_init();
    advertising_init();
    services_init();
    conn_params_init();
    application_timers_start();
		
    err_code = ble_advertising_start(BLE_ADV_MODE_FAST);
		//err_code = ble_advertising_start(BLE_ADV_MODE_IDLE); 
    APP_ERROR_CHECK(err_code);

		

		
		operation_mode=MODE_STBY;


  // Switch ON both RAM banks when in system OFF mode
		NRF_POWER->RAMON |= (POWER_RAMON_OFFRAM0_RAM0On << POWER_RAMON_OFFRAM0_Pos) |
          (POWER_RAMON_OFFRAM1_RAM1On << POWER_RAMON_OFFRAM1_Pos);

    // Start execution.
    NRF_LOG_INFO("Template started\r\n");		
		
    // Enter main loop.
    for (;;)
    {
			
				
        if (NRF_LOG_PROCESS() == false)
        {
            power_manage();
        }
				
    }
}


/**
 * @}
 */
